/* ---------------------------------------------------------------------------- */
/* ClassicSurface                                                               */
/* ---------------------------------------------------------------------------- */

/* Surface proposant la gestion des arêtes à l'aide de matrices d'adjacence et */
/* d'incidence.                                                                 */
/* À noter que les opérations de cette structure de données seront linéaires en */
/* le nombre total de sommêts ou de faces : une alternative peut être           */
/* AdjacencyListSurface pour optimiser certaines opérations.                    */
/* En temps constant pour vérifier qu'une arête est présente ou qu'un sommêt   */
/* est sur une face donnée.                                                     */

/* Pour le moment cette structure n'est plus utilisée. Peut devenir à nouveau   */
/* intéressante pour des opérations qui demandent des vérifications très rapides*/
/* des incidences ou adjacences.                                                */
@Deprecated
class ClassicSurface extends AbstractSurface implements Surface {
  int nF;
  int nV;
  
  boolean[][] adjacency_matrix; /* adjacency_matrix[u][v] == true si l'arête (u, v) existe */
  boolean[][] incidency_matrix; /* incidency_matrix[f][u] == true si u est sur la face f */
  /* Note : ces champs seront alloués dans le constructeur pour une meilleure gestion de la mémoire */
  
  /* change l'orientation d'une face si besoin (selon les orientations des faces précédentes)*/
  private void reverse_face_if_needed(int current_f_id) {
    IntList face = get_face(current_f_id);
    for(int i = 0; i < face.size(); ++i) { /* Pour tous les sommêts de la face */
      int v1 = face.get(i);
      int v2 = face.get((i + 1) % face.size());
      for(int f_id = 0; f_id < current_f_id; ++f_id) { /* on recherche une face précédemment traitée pour vérifier l'orientation à l'aide de l'arête (v1, v2) */
        if(incidency_matrix[f_id][v1] && incidency_matrix[f_id][v2]) {
          IntList other_face = get_face(f_id);
          for(int j = 0; j < other_face.size(); ++j) { /* et corriger l'orientation de la face courante si nécessaire */
            if(other_face.get(j) == v1 && v2 == other_face.get((j + 1) % other_face.size())) {
              face.reverse();
              return;
            }
          }
        }
      }
    }
  }
  
  /* calcul des matrices d'adjacence et d'incidence depuis les données de la surface */
  private void compute_matrices() {
    for(int i = 0; i < nV; ++i) {
      for(int f = 0; f < nF; ++f)
        incidency_matrix[f][i] = false;
      for(int j = 0; j < nV; ++j)
        adjacency_matrix[i][j] = false;
    }
    
    for(int f = 0; f < nF; ++f) {
      IntList face = get_face(f);
      for(int i = 0; i < face.size(); ++i) {
        adjacency_matrix[face.get(i)][face.get((i + 1) % face.size())] = true;
        adjacency_matrix[face.get((i + 1) % face.size())][face.get(i)] = true;
        incidency_matrix[f][face.get(i)] = true;
      }
    }
    for(int f = 0; f < nF; ++f) {
      reverse_face_if_needed(f);
    }
    check_and_change_orientation();
  }
  
  ClassicSurface(SurfaceData data) {
    super(data);
    nV = data.vertices.size();
    nF = data.faces.size();
    adjacency_matrix = new boolean[nV][nV]; /* quadratique en mémoire sur le nombre de sommêts */
    incidency_matrix = new boolean[nF][nV]; /* alloués en fonction de la taillé des données de la surface */
    compute_matrices();
  }
  
  int vertex_degree(int v) {
    int n = 0;
    for(int i = 0; i < nV; ++i)
      if(adjacency_matrix[v][i])
        ++n;
    return n;
  }
  
  IntList neighbors(int p) {
    IntList res = new IntList();
    for(int i = 0; i < nV; ++i) {
      if(adjacency_matrix[p][i])
        res.append(i);
    }
    return res;
  }
  
  IntList faces_in(int p) {
    IntList res = new IntList();
    for(int f = 0; f < nF; ++f) {
      if(incidency_matrix[f][p])
        res.append(f);
    }
    return res;
  }
  
  IntList orientedNeighbors(int p) {
    IntList p_neighbors = neighbors(p);
    IntList p_faces_in = faces_in(p);
    IntList res = new IntList();
    int current_face = p_faces_in.get(0); /* on sélectionne une face sur laquelle on trouve p */
    int last_neighbor = -1;
    
    for(int q : p_neighbors) { /* On cherche un sommet voisin de p sur la face sélectionnée */
      if(incidency_matrix[current_face][q]) {
        last_neighbor = q;
        break;
      }
    }
    res.append(last_neighbor);
    
    while(res.size() < p_neighbors.size()) {
      
      for(int q : p_neighbors) { /* On recherche le voisin suivant : sur la face courrante mais différent de précédent voisin */
        if(incidency_matrix[current_face][q] && q != last_neighbor) {
          last_neighbor = q;
          break;
        }
      }
      
      res.append(last_neighbor);
      
      for(int f : p_faces_in) { /* On recherche la face suivante : telle qu'on avance vers le voisin suivant */
        if(incidency_matrix[f][last_neighbor] && f != current_face) {
          current_face = f;
          break;
        }
      }
    }
    return res;
  }
  
  ArrayList<Edge> fullDataNeighbors(int p) {
    ArrayList<Edge> res = new ArrayList<Edge>();
    for(int v_id : orientedNeighbors(p)) {
      Edge tmp = new Edge(v_id);
      for(int i = 0; i < nF; ++i)
        if(incidency_matrix[i][v_id])
          tmp.add_face_on(i);
      res.add(tmp);
    }
    return res;
  }
  
  void notif_modif_face(int f_id, IntList old_face) {
    /* TODO */
  }
  
  void notif_add_vertex(int v_id) {
    /* TODO */
  }
}
