/* ---------------------------------------------------------------------------- */
/* CurvatureEngine                                                              */
/* ---------------------------------------------------------------------------- */

/* Recette de fabrication du float pour une surface donnée.                     */
/* Interface fonctionnelle : par la suite peut être remplacée par une lambda    */
/* fonction dans une version plus évoluée de java.                              */

@FunctionalInterface
interface CurvatureEngine {
  /* Recette de fabrication de la courbure d'une surface. */
  PVectorList build(Surface surface);
}

/* Dans le cas où la construction du champs de vecteurs se fait depuis une      */
/* courbure sur les arêtes, il est possible d'hériter de la classe             */
/* EdgeCurvatureEngine et ne redéfinir que la méthode build_edge_vectors().     */
/* La méthode build() renverra le champs de vecteurs sur les sommêts depuis le  */
/* champs de vecteurs fabriqué par build_edge_vectors().                        */

abstract class EdgeCurvatureEngine implements CurvatureEngine {
  
  /* Fabrique les champs de vecteur pour une courbure sur les arêtes sous la forme d'une matrice d'adjacence */
  abstract PVector[][] build_edge_vectors(Surface surface);
  
  @Override
  PVectorList build(Surface surface) {
    PVector[][] mat = build_edge_vectors(surface); /* champs de vecteur sur les arêtes */
    PVectorList res = new PVectorList();
    for(int v_id : surface.vertices_ids()) {
      PVector tmp = new PVector(0, 0, 0);
      for(int qi : surface.neighbors(v_id)) /* H(p) est la somme des H(e) tels que p appartient à e */
        tmp.add(mat[v_id][qi]);
      res.add(tmp);
    }
    return res;
  }
}

class GradientCurvature implements CurvatureEngine {  
  @Override
  PVectorList build(Surface surface) {
    PVectorList res = partialGradient(surface);
    for(int v_id : surface.vertices_ids()) {
      PVector tmp = res.get(v_id);
      //tmp = correct_normal(surface, tmp, v_id);
    }
    return res;
  }
}

PVector correct_normal(Surface surface, PVector Np, int p) {
    int ok = 0, no = 0;
    for(int qi : surface.neighbors(p)) {
      if(Np.dot(surface.get(qi, p)) >= 0)
        ++ok;
      else
        ++no;
    }
    if(no > ok)
      Np.mult(-1);
    return Np;
  }
