/* ---------------------------------------------------------------------------- */
/* FacesCurvature                                                               */
/* ---------------------------------------------------------------------------- */

/* Flot donné par le déplacement orthogonal des faces.                          */
/* En cours d'implémentation ...                                                */

class FacesCurvature implements CurvatureEngine {
  PVectorList normals = null;
  PVectorList gradient = null;
  float displace_dist = 1;
  
  private boolean displace_vertex(Surface surface, int v_id, PVectorList res) {
    PVectorList normals = new PVectorList();
    for(int f_id : surface.faces_in(v_id))
      normals.add(this.normals.get(f_id));
    normals.clean_doubles();
    /* considérer les normales évite les cas dégénérés où le sommet est en réalité sur une face ou un arête */
    if(surface.vertex_degree(v_id) <= 3) { /* cas où il existe une solution générique */
      PVector scalars = new PVector(1, 1, 1);
      if(normals.size() == 1) { /* v_id est sur une face, on déplace le sommet dans la direction donnée par la face */
        res.add(PVector.mult(normals.get(0), displace_dist));
        return true;
      } else if(normals.size() == 2) { /* v_id est sur une arête, pour l'intersection on ajoute le plan dont l'arête où se trouve le sommet est la normale */
        PVector tangent = normals.get(0).cross(normals.get(1));
        tangent.normalize();
        normals.add(tangent);
        scalars.z = 0; /* on ne souhaite pas que le sommet se déplace sur l'arête */
      }
      PMatrix normal_mat = new PMatrix(normals);
      PVector move = normal_mat.inverse().mult(scalars).mult(displace_dist);
      res.add(move);
      return true;
    }
    /* Ici nous avons au moins 4 normales différentes */
    PVectorList solutions = new PVectorList();
    for(int i = 0; i < normals.size(); ++i) {
      PMatrix normal_mat = new PMatrix(normals.get(i), normals.get((i + 1) % normals.size()), normals.get((i + 2) % normals.size()));
      PVector move = normal_mat.inverse().mult(new PVector(1, 1, 1)).mult(displace_dist);
      solutions.add(move);
    }
    solutions.clean_doubles();
    if(solutions.size() == 1) { /* si toutes les solutions s'accordent sur le même point on l'ajoute (possible régularité malgré de degré du sommet) */
      res.add(solutions.get(0));
      return true;
    }
    /* Aucune solution ne semble possible dans cette configuration */
    return false;
  }
  
  private void flat_vertex(Surface surface, int v_id, PVectorList res, int original_size) {
    /*PVector vertex_normal = vertex_normal(surface, v_id, gradient);*/
    /* définit un choix de normale pertinent en un sommet ? */
    PVector vertex_normal = gradient.get(v_id);
    int f_id = surface.flat(v_id, vertex_normal);
    IntList face = surface.get_face(f_id);
    
    for(int qi : face) {
      PVectorList normals = new PVectorList();
      for(int fi : surface.faces_in(qi)) {
        if(fi == f_id)
          continue;
        normals.add(this.normals.get(fi));
      }
      /*if(vertex_normal.dot(normals.get(0)) < 0)
        vertex_normal.mult(-1);*/
      normals.add(vertex_normal);
      PVector move = (new PMatrix(normals)).inverse().mult(new PVector(1, 1, 1)).mult(displace_dist);
      if(qi < original_size)
        res.set(qi, move);
      else
        res.add(move);
    }
  }
  
  @Override
  PVectorList build(Surface surface) {
    /*println("START");*/
    ((AbstractSurface)surface).precompute_barycenter_to_ajust_normals();
    gradient = partialGradient(surface);
    PVectorList res = new PVectorList();
    IntList to_flat = new IntList(); /* liste des sommets à écraser */
    normals = surface.faces_normal();
    for(int v_id : surface.vertices_ids()) {
      if(! displace_vertex(surface, v_id, res)) { /* Si aucune solution n'a été trouvée pour ce sommet, il sera transformé en une face l'étape suivante */
        res.add(new PVector(0, 0, 0));
        to_flat.append(v_id);
      }
    }
    int original_size = surface.nV();
    for(int v_id : to_flat) {
      flat_vertex(surface, v_id, res, original_size);
    }
    /*println("modif : vertices : " + surface.nV() + ", faces : " + surface.nF());
    println("res : " + res.size());
    println("END");*/
    ((AbstractSurface)surface).reset_precomputed_barycenter();
    normals = null;
    gradient = null;
    /*for(PVector v : res)
      v.mult(-1);*/
    return res;
  }
}
