/* ---------------------------------------------------------------------------- */
/* Graphic User Interface                                                       */
/* ---------------------------------------------------------------------------- */

/* Permet la gestion d'une interface graphique pour un utilisateur depuis des   */
/* GuiComponents.                                                               */

interface Gui {
  void addComponent(GuiComponent cmp);
  Object get(String name);
  boolean isInGui();
  GuiComponent current();
  
  void mousePressed();
  void draw();
  void mouseReleased();
  
  Iterable<String> values();
}

class AutoOffsetProxyGui implements Gui {
  ArrayGui gui = new ArrayGui();
  PVector offset;
  
  AutoOffsetProxyGui(PVector offset) {
    this.offset = offset;
  }
  
  @Override
  void addComponent(GuiComponent cmp) {
    gui.addComponent(new GuiComponentOffsetProxy(offset, cmp));
    refresh();
  }
  
  @Override
  Object get(String name) {
    return gui.get(name);
  }
  
  @Override
  boolean isInGui() {
    return gui.isInGui();
  }
  
  @Override
  GuiComponent current() {
    return gui.current();
  }
  
  @Override
  void mousePressed() {
    gui.mousePressed();
  }
  
  @Override
  void draw() {
    gui.draw();
  }
  
  @Override
  void mouseReleased() {
    gui.mouseReleased();
  }
  
  @Override
  Iterable<String> values() {
    return gui.values();
  }
  
  void refresh() {
    PVector tmp = offset.copy();
    GuiComponent current;
    for(int i = 0; i < gui.components.size(); ++i) {
      current = ((GuiComponentOffsetProxy)gui.components.get(i)).cmp;
      gui.components.set(i, new GuiComponentOffsetProxy(tmp.copy(), current));
      tmp.add(new PVector(0, (float)current.getHeight()));
    }
  }
}

class ArrayGui implements Gui {
  boolean isInGui = false;
  GuiComponent current = null;
  
  HashMap<String, Object> gui_values = new HashMap<String, Object>(); /* A VarHandle should be a better idea, however it appears only in Java 9 ... */
  ArrayList<GuiComponent> components = new ArrayList<GuiComponent>();
  
  @Override
  void addComponent(GuiComponent cmp) {
    components.add(cmp);
    gui_values.put(cmp.getName(), cmp.getValue());
  }
  
  @Override
  Object get(String name) {
    return gui_values.get(name);
  }
  
  @Override
  boolean isInGui() {
    return isInGui;
  }
  
  @Override
  GuiComponent current() {
    return current;
  }
  
  @Override
  void mousePressed() {
    for(GuiComponent cmp : components) {
      if(cmp.checkIn(mouseX, mouseY)) {
        current = cmp;
        isInGui = true;
        current.getIn(mouseX, mouseY);
        gui_values.put(current.getName(), current.getValue());
      }
    }
  }
  
  @Override
  void draw() {
    for(GuiComponent cmp : components)
      cmp.draw();
    if(isInGui) {
      current.doActions(mouseX, mouseY);
      gui_values.put(current.getName(), current.getValue());
    }
  }
  
  @Override
  void mouseReleased() {
    if(isInGui) {
      current.getOut(mouseX, mouseY);
      gui_values.put(current.getName(), current.getValue());
      isInGui = false;
      current = null;
    }
  }
  
  @Override
  Iterable<String> values() {
    return gui_values.keySet();
  }
}
