/* ---------------------------------------------------------------------------- */
/* HarmonicCurvature                                                            */
/* ---------------------------------------------------------------------------- */

/* Laplacian smoothing sur les surfaces. Le flot de courbure harmonique a pour  */
/* but de lisser au mieux la surface.                                           */
/* Son calcul est donné ici en tout sommêt comme étant la somme des vecteurs    */
/* Qi,P où Qi est voisin de P (Qi appartient à star(P)).                        */

class HarmonicCurvature implements CurvatureEngine {
  boolean probabilistic = false;
  
  HarmonicCurvature() {
  }
  
  HarmonicCurvature(boolean probabilistic) {
    this.probabilistic = probabilistic;
  }
  
  @Override
  PVectorList build(Surface surface) {
    PVectorList res = new PVectorList();
    for(int v_id : surface.vertices_ids()) {
      IntList v = surface.neighbors(v_id);
      PVector tmp = new PVector(0, 0, 0);
      for(int q : v) /* on somme les Qi,P */
        tmp.add(PVector.sub(surface.get(v_id), surface.get(q)));
      if(probabilistic) /* on renormalise par la taille de star(P) pour une version 'probabiliste' */
        tmp.div(v.size());
      res.add(tmp);
    }
    return res;
  }
}
