/* ---------------------------------------------------------------------------- */
/* Tools                                                                        */
/* ---------------------------------------------------------------------------- */

import java.lang.reflect.Array;  

/* Gram-Schmidt générique récupéré de notre code précédent sur les courbes 2D.  */
/* Permet la construction d'une base orthonormée à partir d'une base de champs  */
/* de vecteurs donnés.                                                          */

/* Process Gram-Schmidt algorithm over VectorFields */
/* Here we consider I is VectorField<I> */
/* We expect VectorFields to be PVectorList for example */
<I> ArrayList<VectorField<I>> gramSchmidt(ArrayList<VectorField<I>> V) {
  for(int i = 0; i < V.size() - 1; ++i)
    if(V.get(i).size() != V.get(i + 1).size())
      throw new IllegalArgumentException("Expected same size for VectorFields to apply the Gram-Schmidt algorithm");
      
  ArrayList<VectorField<I>> res = new ArrayList<VectorField<I>>();
  VectorField<I> e;
  VectorField<I>[] basis;
  
  e = V.get(0);
  res.add(e.mult(1 / e.norm()));
  
  for(int i = 1; i < V.size(); ++i) {
    basis = (VectorField<I>[])Array.newInstance(e.getClass(), i);
    for(int j = 0; j < i; ++j)
      basis[j] = res.get(j);
      
    e = V.get(i);
    e = e.sub((I)e.projection((I[])basis));
    res.add(e.mult(1 / e.norm()));
  }
  return res;
}

float cotan(float theta) {
  return cos(theta) / sin(theta);
}

/* Calcule le gradient sur chaque sommêt par la formule donnée pour le gradient partiel */

PVectorList partialGradient(Surface surface) {
  ArrayList<PVector> face_normals = new ArrayList<PVector>();
  for(int f_id = 0; f_id < surface.get_data().faces.size(); ++f_id) {
    face_normals.add(surface.face_normal(f_id));
  }
  PVectorList res = new PVectorList();
  for(int v_id = 0; v_id < surface.get_data().vertices.size(); ++v_id) {
    PVector tmp = new PVector(0, 0, 0);
    for(int f_id : surface.faces_in(v_id)) /* On somme les gradient des différentes faces sur lesquelles on trouve p */
      tmp.add(face_normals.get(f_id));
    tmp.div(6);
    res.add(tmp);
  }
  return res;
}

/* Vérifie si l'arête est dans la même orientation que la face */

boolean edge_oriented_on_face(int v1, int v2, IntList face) {
  for(int i = 0; i < face.size(); ++i)
    if(face.get(i) == v1 && face.get((i + 1) % face.size()) == v2)
      return true;
  return false;
}

<E> ArrayList<E> ArrayList_copy(ArrayList<E> list) {
  ArrayList<E> res = new ArrayList<E>();
  for(E e : list)
    res.add(e);
  return res;
}

/* IntRange propose un itérateur sur les entiers comme dans une boucle for(int i = start; i < end; i += step) */
/* son utilisation a pour vocation d'être utilisé dans une boucle foreach : for(int i : new IntRange(start, end, step)) */

class IntRange implements Iterator<Integer>, Iterable<Integer> {
  int current;
  int end;
  int step;
  
  IntRange(int start, int end, int step) {
    this.current = start;
    this.end = end;
    this.step = step;
  }
  
  IntRange(int start, int end) {
    this(start, end, 1);
  }
  
  IntRange(int end) {
    this(0, end);
  }
  
  @Override
  boolean hasNext() {
    if(step > 0)
      return current < end;
    else
      return current > end;
  }
  
  @Override
  Integer next() {
    int val = current;
    current += step;
    return val;
  }
  
  @Override
  Iterator<Integer> iterator() {
    return this;
  }
}

/* Pour des vérifications rapides sur la norme du vecteur sans avoir à calculer de racine carré */
float norme_carre(PVector v) {
  return v.dot(v);
}
