/* ---------------------------------------------------------------------------- */
/* VectorField                                                                  */
/* ---------------------------------------------------------------------------- */

/* Interface VectorField récupérée de notre code précédent sur les courbes 2D.  */
/* L'intérêt de cette interface est de garantir qu'une classe est équipée d'un  */
/* ensemble de fonctionnalités utiles pour la manipulation de champs de vecteurs*/
/* Ce qui par conséquent permet par exemple de définir une méthode de calcul    */
/* générique pour les champs de vecteurs : exemple l'algorithme de Gram-Schmidt.*/

interface VectorField <I> {
  /* I is the type to interract with, should be itself */
  /* Example : we want to do the scalar product between two PVectorList : so PVectorList should be a VectorField<PVectorList>*/
  /* is used to define a generical way to manipulate VectorFields and do operations with it */
  /* can be use in the Gram-Schmidt algorithm for example */
  
  int size();
  
  float dot(I other); /* scalar product */
  float norm(); /* norm of the VectorField */
  VectorField <I> mult(float scalar); /* multiplication by a scalar value */
  VectorField <I> add(I other); /* addition between two VectorFields */
  VectorField <I> sub(I other); /* substraction between two VectorFields */
  VectorField <I> projection(I... constraint);
  /* get the projection over constraints (force to have at least one constraint) */
  /* use the projection in the curvature would just be Curvature - projection() */
}
